# frozen_string_literal: true

require 'singleton'
require 'logger'
require 'digest'

require 'active_support/deprecation'

module Gitlab
  class Experiment
    class Configuration
      include Singleton

      # Prefix all experiment names with a given value. Use `nil` for none.
      @name_prefix = nil

      # The logger is used to log various details of the experiments.
      @logger = Logger.new($stdout)

      # The base class that should be instantiated for basic experiments.
      @base_class = 'Gitlab::Experiment'

      # The caching layer is expected to respond to fetch, like Rails.cache.
      @cache = nil

      # The domain to use on cookies.
      @cookie_domain = :all

      # The default rollout strategy only works for single variant experiments.
      # It's expected that you use a more advanced rollout for multiple variant
      # experiments.
      @default_rollout = Rollout::Base.new

      # Logic this project uses to determine inclusion in a given experiment.
      # Expected to return a boolean value.
      @inclusion_resolver = lambda do |requested_variant|
        false
      end

      # Tracking behavior can be implemented to link an event to an experiment.
      @tracking_behavior = lambda do |event, args|
        Configuration.logger.info "Gitlab::Experiment[#{name}] #{event}: #{args.merge(signature: signature)}"
      end

      # Called at the end of every experiment run, with the result.
      @publishing_behavior = lambda do |_result|
        track(:assignment)
      end

      # Algorithm that consistently generates a hash key for a given source.
      @context_hash_strategy = lambda do |source, seed|
        source = source.keys + source.values if source.is_a?(Hash)
        data = Array(source).map { |v| (v.respond_to?(:to_global_id) ? v.to_global_id : v).to_s }
        Digest::MD5.hexdigest(data.unshift(seed).join('|'))
      end

      class << self
        # TODO: Added deprecation in release 0.5.0
        def variant_resolver
          ActiveSupport::Deprecation.warn('variant_resolver is deprecated, instead use `inclusion_resolver` with a' \
            'block that returns a boolean.')
          @inclusion_resolver
        end

        def variant_resolver=(block)
          ActiveSupport::Deprecation.warn('variant_resolver is deprecated, instead use `inclusion_resolver` with a' \
            'block that returns a boolean.')
          @inclusion_resolver = block
        end

        attr_accessor(
          :name_prefix,
          :logger,
          :base_class,
          :cache,
          :cookie_domain,
          :default_rollout,
          :inclusion_resolver,
          :tracking_behavior,
          :publishing_behavior,
          :context_hash_strategy
        )
      end
    end
  end
end
